package imgbtn;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

public class ImageButton extends Canvas {

    public static final int UNARMED = 0;
    public static final int ARMED = 1;
    public static final int OVER = 2;
    public static final int DISABLED = 3;

    private static final Border defaultUnarmedBorder =
        new DefaultImageButtonBorder( false );
    private static final Border defaultArmedBorder =
        new DefaultImageButtonBorder( true );

    private MediaTracker tracker;

    private Image images[] = new Image[4];
    private Border borders[] = new Border[4];

    private boolean generatedDisabled = false;
    private boolean mousedown = false;

    private int buttonState = UNARMED;

    private Color backgroundColor=null;

    private String buttonInformation="";

    public ImageButton() {
        tracker = new MediaTracker( this );
        setUnarmedBorder( defaultUnarmedBorder );
        setArmedBorder( defaultArmedBorder );
    }

    public ImageButton( Image image) {
        this();
        enableEvents(AWTEvent.MOUSE_EVENT_MASK);
        setUnarmedImage( image );
        setBackground(backgroundColor);
    }

    public ImageButton( Image image, String info) {
        this();
        enableEvents(AWTEvent.MOUSE_EVENT_MASK);
        setUnarmedImage( image );
        buttonInformation=info;
        setBackground(backgroundColor);
    }

	private synchronized void setImage( int id, Image image ) {
	    if ( images[id] != image ) {
       	    images[id] = image;
        	if ( image != null ) {
    	        tracker.addImage( image, id );
        	    tracker.checkID( id, true );
        	}
    	    if ( buttonState == id ) {
        	    repaint();
        	}
        }
	}

	public void setUnarmedImage( Image image ) {
        setImage( UNARMED, image );
        if ( images[ARMED] == null ) {
            setArmedImage( image );
        }
        if ( images[OVER] == null ) {
            setOverImage( image );
        }
        if ( ( images[DISABLED] == null ) ||
             generatedDisabled ) {
            setDisabledImage( null );
        }
	}

	public void setArmedImage( Image image ) {
   	    if ( image != null ) {
       	    setImage( ARMED, image );
   	    }
   	    else {
   	        setImage( ARMED, images[UNARMED] );
   	    }
	}

	public void setOverImage( Image image ) {
   	    if ( image != null ) {
       	    setImage( OVER, image );
   	    }
   	    else {
   	        setImage( OVER, images[UNARMED] );
   	    }
	}

	public void setDisabledImage( Image image ) {
        generatedDisabled = false;
   	    if ( ( image == null ) &&
   	         ( images[UNARMED] != null ) ) {
            generatedDisabled = true;
            image = createImage( new FilteredImageSource( images[UNARMED].getSource(),
                                                          new DisableImageFilter() ) );
   	    }
	    setImage( DISABLED, image );
	}

	public Image getUnarmedImage() {
		return ( images[UNARMED] );
	}

	public Image getArmedImage() {
		return ( images[ARMED] );
	}

	public Image getOverImage() {
		return ( images[OVER] );
	}

	public Image getDisabledImage() {
		return ( images[DISABLED] );
	}

	private synchronized void setBorder( int id, Border border ) {
	    if ( borders[id] != border ) {
       	    borders[id] = border;
       	    if ( buttonState == id ) {
       	        repaint();
       	    }
        }
	}

	public void setUnarmedBorder( Border border ) {
        setBorder( UNARMED, border );
        if ( borders[ARMED] == null ) {
            setArmedBorder( border );
        }
        if ( borders[OVER] == null ) {
            setOverBorder( border );
        }
        if ( borders[DISABLED] == null ) {
            setDisabledBorder( border );
        }
	}

	public void setArmedBorder( Border border ) {
   	    if ( border != null ) {
       	    setBorder( ARMED, border );
   	    }
   	    else {
   	        setBorder( ARMED, borders[UNARMED] );
   	    }
	}

	public void setOverBorder( Border border ) {
   	    if ( border != null ) {
       	    setBorder( OVER, border );
   	    }
   	    else {
   	        setBorder( OVER, borders[UNARMED] );
   	    }
	    setBorder( OVER, border );
	}

	public void setDisabledBorder( Border border ) {
   	    if ( border != null ) {
       	    setBorder( DISABLED, border );
   	    }
   	    else {
   	        setBorder( DISABLED, borders[UNARMED] );
   	    }
	    if ( buttonState == DISABLED ) {
    	    repaint();
    	}
	}

	public Border getUnarmedBorder() {
		return ( borders[UNARMED] );
	}

	public Border getArmedBorder() {
		return ( borders[ARMED] );
	}

	public Border getOverBorder() {
		return ( borders[OVER] );
	}

	public Border getDisabledBorder() {
		return ( borders[DISABLED] );
	}

	public int getButtonState() {
	    return buttonState;
	}

	protected void setButtonState( int buttonState ) {
	    if ( buttonState != this.buttonState ) {
    		this.buttonState = buttonState;
            repaint();
    	}
	}


    public void setEnabled(boolean b){
        if(b) setButtonState( UNARMED );
        else setButtonState( DISABLED );
        super.setEnabled(b);
    }

    public void paint( Graphics g ) {
        Dimension size = getSize();
        g.setColor(backgroundColor);
        borders[buttonState].paint( g, null/*getBackground()*/, 0, 0, size.width, size.height );
        try {
            if ( ! tracker.checkID( buttonState ) ) {
                tracker.waitForID( buttonState );
            }
            if ( ! tracker.isErrorID( buttonState ) ) {
                Insets insets = borders[buttonState].getInsets();
                int imageWidth = images[buttonState].getWidth( this );
                int imageHeight = images[buttonState].getHeight( this );
                int x = insets.left +
                        ( ( ( size.width - ( insets.left + insets.right ) ) -
                            imageWidth ) / 2 );
                int y = insets.top +
                        ( ( ( size.height - ( insets.top + insets.bottom ) ) -
                            imageHeight ) / 2 );
                g.drawImage( images[buttonState], x, y, this );
            }
        }
        catch ( InterruptedException ie ) {
        }
    }

    public Dimension getPreferredSize() {
        Dimension pref = new Dimension();
        try {
            if ( ! tracker.checkID( buttonState ) ) {
                tracker.waitForID( buttonState );
            }
            if ( ! tracker.isErrorID( buttonState ) ) {
                Dimension size = getSize();
                pref.width = images[buttonState].getWidth( this );
                pref.height = images[buttonState].getHeight( this );
            }
            int maxWidthAdd = 0;
            int maxHeightAdd = 0;
            for ( int i = 0; i < DISABLED; i++ ) {
                Insets insets = borders[i].getInsets();
                maxWidthAdd = Math.max( maxWidthAdd, insets.left+insets.right );
                maxHeightAdd = Math.max( maxHeightAdd, insets.top+insets.bottom );
            }
            pref.width += maxWidthAdd;
            pref.height += maxHeightAdd;
        }
        catch ( InterruptedException ie ) {
        }
        return pref;
    }



  protected void processMouseEvent(MouseEvent event){

    switch(event.getID()){
      case MouseEvent.MOUSE_PRESSED:
        mousedown = true;
        setButtonState( ARMED );
        break;

      case MouseEvent.MOUSE_RELEASED:
        mousedown = false;
        if ( contains( event.getX(), event.getY() ) ) setButtonState( OVER );
        break;

      case MouseEvent.MOUSE_ENTERED:
        if ( mousedown ) setButtonState( ARMED );
        else setButtonState( OVER );
        break;

      case MouseEvent.MOUSE_EXITED:
        setButtonState( UNARMED );
        break;
    }
    super.processMouseEvent(event);
  }

    public String getMessage(){
        return buttonInformation;
    }
}

